//+------------------------------------------------------------------+
//|                                              PriceActionEA.mq5   |
//|                        Price Action EA with HTF Analysis         |
//|                        Integrated with MTC Combo v2              |
//|                        and Conservative Strategy                 |
//+------------------------------------------------------------------+
#property strict

// Input parameters from PriceActionEA
input double StopLossPips = 50;          // Stop Loss in pips
input double TakeProfitPips = 100;         // Take Profit in pips
input double LotSize = 0.1;               // Lot size
input int MA_Period = 10;                 // Moving Average period
input int RSI_Period = 14;                // RSI period
input ENUM_TIMEFRAMES SelectedTimeFrame = PERIOD_M15; // Selected timeframe
input int SelectedStrategy = 1;           // 1 = Aggressive, 2 = Trend Following, 3 = Hammer/Shooting Star, 4 = Double bottom Double top , 5 = Following Trend Strategy
input bool WaitForConfirmation = false;   // Wait for confirmation candle
input double FibonacciLevel = 38.2;       // Fibonacci level for patterns
input double TerminationZoneSize = 10;    // Termination zone size in pips
input int ATRPeriod = 14;                 // ATR period for volatility filter
input int ADXThreshold = 25;              // ADX threshold for trend strength
input bool AvoidNewsEvents = true;        // Avoid trading during news events
input bool UseTradingHours = true;        // Limit trading to specific hours
input datetime TradingStartTime = D'2025.2.01 08:00'; // Trading start time
input datetime TradingEndTime = D'2025.2.01 16:00';   // Trading end time
input bool EnableNotifications = true;    // Enable notifications

// Input parameters from MTC Combo v2
input uchar              bar = 0;         // Bar index for MA
input int                ma_period = 2;   // Averaging period for MA
input int                ma_shift = 0;    // Horizontal shift for MA
input ENUM_MA_METHOD     ma_method = MODE_SMA; // Smoothing type for MA
input ENUM_APPLIED_PRICE applied_price = PRICE_CLOSE; // Price type for MA
input double      tp1 = 50;               // Take Profit 1
input double      sl1 = 50;               // Stop Loss 1
input int         x12 = 100;              // Weight for perceptron1
input int         x22 = 100;              // Weight for perceptron1
input int         x32 = 100;              // Weight for perceptron1
input int         x42 = 100;              // Weight for perceptron1
input double      tp2 = 50;               // Take Profit 2
input double      sl2 = 50;               // Stop Loss 2
input int         p2 = 20;                // Period for perceptron1
input int         x13 = 100;              // Weight for perceptron2
input int         x23 = 100;              // Weight for perceptron2
input int         x33 = 100;              // Weight for perceptron2
input int         x43 = 100;              // Weight for perceptron2
input double      tp3 = 50;               // Take Profit 3
input double      sl3 = 50;               // Stop Loss 3
input int         p3 = 20;                // Period for perceptron2
input int         x14 = 100;              // Weight for perceptron3
input int         x24 = 100;              // Weight for perceptron3
input int         x34 = 100;              // Weight for perceptron3
input int         x44 = 100;              // Weight for perceptron3
input int         p4 = 20;                // Period for perceptron3
input int         pass = 10;              // Strategy complexity level
input ulong       mn = 888;               // Magic number

// Input parameters for perceptron improvements
input int RSI_Period_Perceptron = 14;     // Period for RSI in perceptron
input int ATR_Period_Perceptron = 14;     // Period for ATR in perceptron
input int MACD_Fast = 12;                 // Fast period for MACD
input int MACD_Slow = 26;                 // Slow period for MACD
input int MACD_Signal = 9;                // Signal period for MACD
input int x52 = 100; // Weight for volume in perceptron
input int x62 = 100; // Weight for RSI in perceptron
input int x72 = 100; // Weight for ATR in perceptron
input int x82 = 100; // Weight for MACD in perceptron

// Input parameters for Conservative Strategy
input double RiskRewardRatio = 1.4; // Reward-to-risk ratio
input int MinCandlesBetweenBottoms = 5; // Minimum candles between bottoms/tops

// Global variables
double SwingHighs[2];
double SwingLows[2];
bool hasTwoSwingHighs = false;
bool hasTwoSwingLows = false;
int TradeLogIndex = 0;
string TradeLog[100];
bool EA_Active = true;
static datetime prevtime = 0;
static double m_sl = 100;
static double m_tp = 100;

// Global variables for Conservative Strategy
double terminationZoneHigh = 0.0;
double terminationZoneLow = 0.0;
bool isDoubleBottom = false;
bool isDoubleTop = false;

//+------------------------------------------------------------------+
//| Global Variables for Following Trend Strategy                    |
//+------------------------------------------------------------------+
double majorSwingHigh = 0.0; // Stores the major swing high
double majorSwingLow = 0.0;  // Stores the major swing low
bool isUptrend = false;      // Flag to indicate an uptrend
bool isDowntrend = false;    // Flag to indicate a downtrend
double optimalZoneHigh = 0.0; // Optimal trading zone for uptrends
double optimalZoneLow = 0.0;  // Optimal trading zone for downtrends

// Include Trade library
#include <Trade\Trade.mqh>
#include <Trade\AccountInfo.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\PositionInfo.mqh>
CTrade trade;
CAccountInfo m_account;
CSymbolInfo m_symbol;
CPositionInfo m_position;

// Indicator handles
int handle_iMA;


///+------------------------------------------------------------------+
//| Analyze Higher Time Frame (HTF) for Trend and Structure Levels   |
//+------------------------------------------------------------------+
bool AnalyzeHTF(ENUM_TIMEFRAMES htf, double &supportLevel, double &resistanceLevel)
{
    // Determine trend direction using MA on HTF
    int htfMAHandle = iMA(_Symbol, htf, MA_Period, ma_shift, ma_method, applied_price);
    double htfMA[];
    ArraySetAsSeries(htfMA, true);
    if (CopyBuffer(htfMAHandle, 0, 0, 2, htfMA) <= 0)
    {
        Print("Failed to copy HTF MA buffer");
        return false;
    }

    // Determine trend direction (use local variables with different names)
    bool localIsUptrend = htfMA[0] > htfMA[1]; // Local variable
    bool localIsDowntrend = htfMA[0] < htfMA[1]; // Local variable

    // Update global variables if needed
    isUptrend = localIsUptrend; // Update global isUptrend
    isDowntrend = localIsDowntrend; // Update global isDowntrend

    // Identify key structure levels (support and resistance)
    supportLevel = iLow(_Symbol, htf, iLowest(_Symbol, htf, MODE_LOW, 50, 0));
    resistanceLevel = iHigh(_Symbol, htf, iHighest(_Symbol, htf, MODE_HIGH, 50, 0));

    // Check if support and resistance levels are valid
    if (supportLevel <= 0 || resistanceLevel <= 0)
    {
        Print("Invalid support or resistance level detected.");
        return false;
    }

    // Add ATR filter to avoid false signals in low volatility
    int htfATRHandle = iATR(_Symbol, htf, ATRPeriod);
    double htfATR[];
    ArraySetAsSeries(htfATR, true);
    if (CopyBuffer(htfATRHandle, 0, 0, 1, htfATR) <= 0)
    {
        Print("Failed to copy HTF ATR buffer");
        return false;
    }

    // Check if ATR is above a threshold (e.g., 10 pips)
    if (htfATR[0] < 10 * _Point)
    {
        Print("Low volatility detected on HTF. Skipping trade.");
        return false;
    }

    return localIsUptrend || localIsDowntrend;
}
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int handle_iRSI;
int handle_iATR;
int handle_iMACD;
int handle_iADX;

int OnInit()
{
    // Create handle for MA indicator
    handle_iMA = iMA(Symbol(), Period(), ma_period, ma_shift, ma_method, applied_price);
    if (handle_iMA == INVALID_HANDLE)
    {
        Print("Failed to create handle for MA indicator");
        return INIT_FAILED;
    }

    // Create handle for RSI indicator
    handle_iRSI = iRSI(Symbol(), Period(), RSI_Period_Perceptron, PRICE_CLOSE);
    if (handle_iRSI == INVALID_HANDLE)
    {
        Print("Failed to create handle for RSI indicator");
        return INIT_FAILED;
    }

    // Create handle for ATR indicator
    handle_iATR = iATR(Symbol(), Period(), ATR_Period_Perceptron);
    if (handle_iATR == INVALID_HANDLE)
    {
        Print("Failed to create handle for ATR indicator");
        return INIT_FAILED;
    }

    // Create handle for MACD indicator
    handle_iMACD = iMACD(Symbol(), Period(), MACD_Fast, MACD_Slow, MACD_Signal, PRICE_CLOSE);
    if (handle_iMACD == INVALID_HANDLE)
    {
        Print("Failed to create handle for MACD indicator");
        return INIT_FAILED;
    }

    // Create handle for ADX indicator
    handle_iADX = iADX(Symbol(), Period(), ADXThreshold);
    if (handle_iADX == INVALID_HANDLE)
    {
        Print("Failed to create handle for ADX indicator");
        return INIT_FAILED;
    }

    // Initialize trading objects
    trade.SetExpertMagicNumber(mn);
    trade.SetDeviationInPoints(10);
    m_symbol.Name(Symbol());
    m_symbol.RefreshRates();

    Print("Price Action EA initialized successfully.");
    return INIT_SUCCEEDED;
}
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
    ObjectsDeleteAll(0);
    Print("Price Action EA deinitialized.");
}
//+------------------------------------------------------------------+
//| Check for Major Swing Highs and Lows                             |
//+------------------------------------------------------------------+
void CheckSwingHighsLows()
{
    // Check for uptrend (higher highs and higher lows)
    if (iClose(_Symbol, PERIOD_CURRENT, 1) > iClose(_Symbol, PERIOD_CURRENT, 2) &&
        iLow(_Symbol, PERIOD_CURRENT, 1) > iLow(_Symbol, PERIOD_CURRENT, 2))
    {
        // Check for at least 3 red candles in pullback
        int redCandles = 0;
        for (int i = 0; i <= 3; i++)
        {
            if (iClose(_Symbol, PERIOD_CURRENT, i) < iOpen(_Symbol, PERIOD_CURRENT, i))
                redCandles++;
        }
        if (redCandles >= 3)
        {
            majorSwingHigh = iHigh(_Symbol, PERIOD_CURRENT, 1);
            isUptrend = true;
            isDowntrend = false;
            
             // Debug: Stampa gli swing high e swing low
    Print("Swing High: ", majorSwingHigh);
    Print("Swing Low: ", majorSwingLow);
        }
    }

    // Check for downtrend (lower highs and lower lows)
    if (iClose(_Symbol, PERIOD_CURRENT, 1) < iClose(_Symbol, PERIOD_CURRENT, 2) &&
        iHigh(_Symbol, PERIOD_CURRENT, 1) < iHigh(_Symbol, PERIOD_CURRENT, 2))
    {
        // Check for at least 3 green candles in pullback
        int greenCandles = 0;
        for (int i = 0; i <= 3; i++)
        {
            if (iClose(_Symbol, PERIOD_CURRENT, i) > iOpen(_Symbol, PERIOD_CURRENT, i))
                greenCandles++;
        }
        if (greenCandles >= 3)
        {
            majorSwingLow = iLow(_Symbol, PERIOD_CURRENT, 1);
            isDowntrend = true;
            isUptrend = false;
        }
    }
}

//+------------------------------------------------------------------+
//| Check for Optimal Trading Zones                                  |
//+------------------------------------------------------------------+
void CheckOptimalTradingZones()
{
    if (isUptrend)
    {
        // Optimal trading zone for uptrend: near previous major swing high
        optimalZoneHigh = majorSwingHigh;
        optimalZoneLow = 0.0; // Reset for downtrend
    }
    else if (isDowntrend)
    {
        // Optimal trading zone for downtrend: near previous major swing low
        optimalZoneLow = majorSwingLow;
        optimalZoneHigh = 0.0; // Reset for uptrend
    }
}

//+------------------------------------------------------------------+
//| Check for Entry Patterns (Close Above/Below Candle)              |
//+------------------------------------------------------------------+
bool CheckEntryPatterns()
{
    // Close Above Candle (Bullish Entry)
    if (isUptrend && iClose(_Symbol, PERIOD_CURRENT, 0) > iHigh(_Symbol, PERIOD_CURRENT, 1) &&
        iClose(_Symbol, PERIOD_CURRENT, 0) > iOpen(_Symbol, PERIOD_CURRENT, 0))
    {
        return true; // Bullish entry signal
    }

    // Close Below Candle (Bearish Entry)
    if (isDowntrend && iClose(_Symbol, PERIOD_CURRENT, 0) < iLow(_Symbol, PERIOD_CURRENT, 1) &&
        iClose(_Symbol, PERIOD_CURRENT, 0) < iOpen(_Symbol, PERIOD_CURRENT, 0))
    {
        return true; // Bearish entry signal
    }

    return false; // No entry signal
}

//+------------------------------------------------------------------+
//| Execute Full Strategy                                            |
//+------------------------------------------------------------------+
void ExecuteFullStrategy()
{
    // Step 1: Check for major swing highs and lows
    CheckSwingHighsLows();

    // Step 2: Check for optimal trading zones
    CheckOptimalTradingZones();

    // Step 3: Check for entry patterns
    if (CheckEntryPatterns())
    {
        if (isUptrend)
        {
            // Bullish trade: Buy with stop loss below swing low and take profit at 1.4:1 RR
            double entryPrice = iClose(_Symbol, PERIOD_CURRENT, 0);
            double stopLoss = iLow(_Symbol, PERIOD_CURRENT, 1) - (10 * _Point); // Stop loss below swing low
            double takeProfit = entryPrice + (1.4 * (entryPrice - stopLoss));   // 1.4:1 reward-to-risk ratio

            trade.Buy(LotSize, _Symbol, entryPrice, stopLoss, takeProfit, "Bullish Trend Continuation");
        }
        else if (isDowntrend)
        {
            // Bearish trade: Sell with stop loss above swing high and take profit at 1.4:1 RR
            double entryPrice = iClose(_Symbol, PERIOD_CURRENT, 0);
            double stopLoss = iHigh(_Symbol, PERIOD_CURRENT, 1) + (10 * _Point); // Stop loss above swing high
            double takeProfit = entryPrice - (1.4 * (stopLoss - entryPrice));    // 1.4:1 reward-to-risk ratio

            trade.Sell(LotSize, _Symbol, entryPrice, stopLoss, takeProfit, "Bearish Trend Continuation");
        }
    }
}
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
// Funzione avanzata per verificare i livelli chiave di struttura con filtri migliorati
bool IsKeyStructureLevel(int type, ENUM_TIMEFRAMES timeframe, double buffer) {
    // Ottieni il minimo e il massimo delle ultime 50 candele sul timeframe specificato
    double support = iLow(_Symbol, timeframe, iLowest(_Symbol, timeframe, MODE_LOW, 450, 0));
    double resistance = iHigh(_Symbol, timeframe, iHighest(_Symbol, timeframe, MODE_HIGH, 450, 0));

    // Verifica che i valori di supporto e resistenza siano validi
    if (support == 0.0 || resistance == 0.0) {
        Print("Errore: supporto o resistenza non validi.");
        return false;
    }

    // Ottieni il prezzo di chiusura della candela corrente
    double closePrice = iClose(_Symbol, PERIOD_CURRENT, 0);

    // Aggiungi un buffer per evitare falsi segnali
    support += buffer;
    resistance -= buffer;

    // Debug: Stampa i livelli di supporto e resistenza
    Print("Supporto: ", support, " | Resistenza: ", resistance, " | Prezzo di chiusura: ", closePrice);

    // Verifica i livelli di struttura con tolleranza
    if (type == ORDER_TYPE_BUY) {
        return closePrice > support; // Prezzo sopra il supporto con margine
    } else if (type == ORDER_TYPE_SELL) {
        return closePrice < resistance; // Prezzo sotto la resistenza con margine
    }
    return false;
}

// Funzione per verificare il trend
bool IsValidTrend(int type) {
    // Ottieni la media mobile
    double ma = iMA(_Symbol, PERIOD_CURRENT, MA_Period, 0, MODE_SMA, PRICE_CLOSE);
    // Ottieni il prezzo di chiusura della candela corrente
    double closePrice = iClose(_Symbol, PERIOD_CURRENT, 0);
    // Verifica il trend
    if (type == ORDER_TYPE_BUY) {
        return closePrice > ma; // Uptrend
    } else if (type == ORDER_TYPE_SELL) {
        return closePrice < ma; // Downtrend
    }
    return false;
}

void OnTick() 
{
    // Controllo se l'EA è attivo
    if (!EA_Active) {
        Print("EA is not active.");
        return;
    }

    // Controllo della volatilità
    if (!IsVolatilityAcceptable()) {
        Print("Volatility is not acceptable.");
        return;
    }

    // Controllo della forza del trend
    if (!IsTrendStrong()) {
        Print("Trend is not strong.");
        return;
    }

    // Controllo degli eventi di news
    if (AvoidNewsEvents && IsNewsEventActive()) {
        Print("News event is active.");
        return;
    }

    // Controllo degli orari di trading
    if (UseTradingHours && !IsTradingTime()) {
        Print("Not in trading hours.");
        return;
    }

    // Analisi del timeframe superiore (HTF) per trend e livelli di struttura
    double htfSupport, htfResistance;
    ENUM_TIMEFRAMES htf = GetHigherTimeFrame(SelectedTimeFrame);
    if (!AnalyzeHTF(htf, htfSupport, htfResistance)) {
        Print("Failed to analyze HTF.");
        return;
    }
    // Esegui la strategia completa
    ExecuteFullStrategy();
    
    // Decisione del perceptron
    double decision = Supervisor();
    Print("Decision: ", decision);

    // Apertura di un'operazione di acquisto (BUY)
    if (decision > 0 && OrdersTotal() == 0 && htfSupport > 0) {
        Print("Opening Buy Trade.");
        OpenTrade(ORDER_TYPE_BUY);
    }
    // Apertura di un'operazione di vendita (SELL)
    else if (decision < 0 && OrdersTotal() == 0 && htfResistance > 0) {
        Print("Opening Sell Trade.");
        OpenTrade(ORDER_TYPE_SELL);
    }

    // Verifica dei pattern Double Bottom o Double Top
    if (CheckDoubleBottom() || CheckDoubleTop()) {
        Print("Double Bottom/Top detected.");
        OpenConservativeTrade();
    }

   // Verifica del pattern Hammer (BUY)
if (IsHammer() && IsValidTrend(ORDER_TYPE_BUY) && IsKeyStructureLevel(ORDER_TYPE_BUY, PERIOD_CURRENT, 5 * _Point)) {
    if (!WaitForConfirmation || IsConfirmationCandle(ORDER_TYPE_BUY)) {
        Print("Hammer detected, opening Buy Trade.");
        OpenTrade(ORDER_TYPE_BUY);
    } // Aggiunto questa parentesi graffa chiusa
}
    // Copia il buffer per RSI
    double rsiBuffer[];
    ArraySetAsSeries(rsiBuffer, true);
    int rsiHandle = iRSI(_Symbol, PERIOD_CURRENT, RSI_Period, PRICE_CLOSE);
    if (CopyBuffer(rsiHandle, 0, 0, 1, rsiBuffer) <= 0) {
    Print("Errore durante la copia del buffer RSI: ", GetLastError());
    return; // Interrompe l'esecuzione della funzione
    }

    // Copia il buffer per ATR
    double atrBuffer[];
    ArraySetAsSeries(atrBuffer, true);
    if (CopyBuffer(handle_iATR, 0, 0, 1, atrBuffer) <= 0)
    {
        Print("Failed to copy ATR buffer");
        return;
    }

    // Copia il buffer per MACD
    double macdBuffer[];
    ArraySetAsSeries(macdBuffer, true);
    if (CopyBuffer(handle_iMACD, 0, 0, 1, macdBuffer) <= 0)
    {
        Print("Failed to copy MACD buffer");
        return;
    }

    // Copia il buffer per ADX
    double adxBuffer[];
    ArraySetAsSeries(adxBuffer, true);
    if (CopyBuffer(handle_iADX, 0, 0, 1, adxBuffer) <= 0)
    {
        Print("Failed to copy ADX buffer");
        return;
    }

    // Verifica del pattern Shooting Star (SELL)
    if (IsShootingStar() && IsValidTrend(ORDER_TYPE_SELL) && IsKeyStructureLevel(ORDER_TYPE_SELL, PERIOD_CURRENT, 5 * _Point)) {
        if (!WaitForConfirmation || IsConfirmationCandle(ORDER_TYPE_SELL)) {
            Print("Shooting Star detected, opening Sell Trade.");
            OpenTrade(ORDER_TYPE_SELL);
        }
      
  } 
} // Questa parentesi chiude correttamente la funzione OnTick()
 

void DrawFibonacci() {
    // Ottieni i prezzi massimi e minimi della candela corrente
    double highPrice = iHigh(_Symbol, PERIOD_CURRENT, 0);
    double lowPrice = iLow(_Symbol, PERIOD_CURRENT, 0);

    // Crea i livelli di Fibonacci
    int fibHandle = ObjectCreate(0, "FibLevels", OBJ_FIBO, 0, TimeCurrent(), highPrice, TimeCurrent(), lowPrice);
    if (fibHandle != INVALID_HANDLE) {
        ObjectSetInteger(0, "FibLevels", OBJPROP_LEVELCOLOR, clrBlue);
        ObjectSetInteger(0, "FibLevels", OBJPROP_LEVELSTYLE, STYLE_SOLID);
        ObjectSetDouble(0, "FibLevels", OBJPROP_LEVELVALUE, 0, 0.0); // 0%
        ObjectSetDouble(0, "FibLevels", OBJPROP_LEVELVALUE, 1, 0.382); // 38.2%
        ObjectSetDouble(0, "FibLevels", OBJPROP_LEVELVALUE, 2, 0.5); // 50%
        ObjectSetDouble(0, "FibLevels", OBJPROP_LEVELVALUE, 3, 0.618); // 61.8%
        ObjectSetDouble(0, "FibLevels", OBJPROP_LEVELVALUE, 4, 1.0); // 100%
    }
}
//+------------------------------------------------------------------+
//| Get Higher Time Frame                                            |
//+------------------------------------------------------------------+
ENUM_TIMEFRAMES GetHigherTimeFrame(ENUM_TIMEFRAMES ltf)
{
    switch(ltf)
    {
        case PERIOD_M1:  return PERIOD_M5;
        case PERIOD_M5:  return PERIOD_M15;
        case PERIOD_M15: return PERIOD_H1;
        case PERIOD_H1: return PERIOD_H4;
        case PERIOD_H4: return PERIOD_D1;
        case PERIOD_D1: return PERIOD_W1;
        case PERIOD_W1: return PERIOD_MN1;
        default: return PERIOD_H1;
    }
}

//+------------------------------------------------------------------+
//| Supervisor function from MTC Combo v2                            |
//+------------------------------------------------------------------+
double Supervisor()
{
    if (pass == 4)
    {
        if (perceptron3() > 0)
        {
            if (perceptron2() > 0)
            {
                m_sl = sl3;
                m_tp = tp3;
                return 1.0;
            }
        }
        else
        {
            if (perceptron1() < 0)
            {
                m_sl = sl2;
                m_tp = tp2;
                return -1.0;
            }
        }
        return basicTradingSystem();
    }

    if (pass == 3)
    {
        if (perceptron2() > 0)
        {
            m_sl = sl3;
            m_tp = tp3;
            return 1.0;
        }
        else
        {
            return basicTradingSystem();
        }
    }

    if (pass == 2)
    {
        if (perceptron1() < 0)
        {
            m_sl = sl2;
            m_tp = tp2;
            return -1.0;
        }
        else
        {
            return basicTradingSystem();
        }
    }
    return basicTradingSystem();
}

//+------------------------------------------------------------------+
//| Perceptron functions with additional inputs                      |
//+------------------------------------------------------------------+
double perceptron1() {
    double w1 = x12 - 100;
    double w2 = x22 - 100;
    double w3 = x32 - 100;
    double w4 = x42 - 100;
    double w5 = x52 - 100; // Weight for volume
    double w6 = x62 - 100; // Weight for RSI
    double w7 = x72 - 100; // Weight for ATR
    double w8 = x82 - 100; // Weight for MACD

    // Inputs
    double a1 = iClose(Symbol(), Period(), 0) - iOpen(Symbol(), Period(), p2);
    double a2 = iOpen(Symbol(), Period(), p2) - iOpen(Symbol(), Period(), p2 * 2);
    double a3 = iOpen(Symbol(), Period(), p2 * 2) - iOpen(Symbol(), Period(), p2 * 3);
    double a4 = iOpen(Symbol(), Period(), p2 * 3) - iOpen(Symbol(), Period(), p2 * 4);
    double a5 = (double)iVolume(Symbol(), Period(), 0); // Volume (conversione esplicita)
    double a6 = iRSI(Symbol(), Period(), RSI_Period_Perceptron, PRICE_CLOSE); // RSI
    double a7 = iATR(Symbol(), Period(), ATR_Period_Perceptron); // ATR
    double a8 = iMACD(Symbol(), Period(), MACD_Fast, MACD_Slow, MACD_Signal, PRICE_CLOSE); // MACD

    return w1 * a1 + w2 * a2 + w3 * a3 + w4 * a4 + w5 * a5 + w6 * a6 + w7 * a7 + w8 * a8;
}

double perceptron2() {
    double w1 = x12 - 100;
    double w2 = x22 - 100;
    double w3 = x32 - 100;
    double w4 = x42 - 100;
    double w5 = x52 - 100; // Weight for volume
    double w6 = x62 - 100; // Weight for RSI
    double w7 = x72 - 100; // Weight for ATR
    double w8 = x82 - 100; // Weight for MACD

    // Inputs
    double a1 = iClose(Symbol(), Period(), 0) - iOpen(Symbol(), Period(), p2);
    double a2 = iOpen(Symbol(), Period(), p2) - iOpen(Symbol(), Period(), p2 * 2);
    double a3 = iOpen(Symbol(), Period(), p2 * 2) - iOpen(Symbol(), Period(), p2 * 3);
    double a4 = iOpen(Symbol(), Period(), p2 * 3) - iOpen(Symbol(), Period(), p2 * 4);
    double a5 = (double)iVolume(Symbol(), Period(), 0); // Volume (conversione esplicita)
    double a6 = iRSI(Symbol(), Period(), RSI_Period_Perceptron, PRICE_CLOSE); // RSI
    double a7 = iATR(Symbol(), Period(), ATR_Period_Perceptron); // ATR
    double a8 = iMACD(Symbol(), Period(), MACD_Fast, MACD_Slow, MACD_Signal, PRICE_CLOSE); // MACD

    return w1 * a1 + w2 * a2 + w3 * a3 + w4 * a4 + w5 * a5 + w6 * a6 + w7 * a7 + w8 * a8;
}

double perceptron3() {
    double w1 = x12 - 100;
    double w2 = x22 - 100;
    double w3 = x32 - 100;
    double w4 = x42 - 100;
    double w5 = x52 - 100; // Weight for volume
    double w6 = x62 - 100; // Weight for RSI
    double w7 = x72 - 100; // Weight for ATR
    double w8 = x82 - 100; // Weight for MACD

    // Inputs
    double a1 = iClose(Symbol(), Period(), 0) - iOpen(Symbol(), Period(), p2);
    double a2 = iOpen(Symbol(), Period(), p2) - iOpen(Symbol(), Period(), p2 * 2);
    double a3 = iOpen(Symbol(), Period(), p2 * 2) - iOpen(Symbol(), Period(), p2 * 3);
    double a4 = iOpen(Symbol(), Period(), p2 * 3) - iOpen(Symbol(), Period(), p2 * 4);
    double a5 = (double)iVolume(Symbol(), Period(), 0); // Volume (conversione esplicita)
    double a6 = iRSI(Symbol(), Period(), RSI_Period_Perceptron, PRICE_CLOSE); // RSI
    double a7 = iATR(Symbol(), Period(), ATR_Period_Perceptron); // ATR
    double a8 = iMACD(Symbol(), Period(), MACD_Fast, MACD_Slow, MACD_Signal, PRICE_CLOSE); // MACD

    return w1 * a1 + w2 * a2 + w3 * a3 + w4 * a4 + w5 * a5 + w6 * a6 + w7 * a7 + w8 * a8;
}

//+------------------------------------------------------------------+
//| Multi-Layer Perceptron (MLP) implementation                      |
//+------------------------------------------------------------------+
double MLP(double &inputs[], double &weights_hidden[], double &weights_output[], int input_size, int hidden_size) {
    // Hidden layer calculation
    double hidden[];
    ArrayResize(hidden, hidden_size); // Dynamically allocate array size

    for (int i = 0; i < hidden_size; i++) {
        hidden[i] = 0;
        for (int j = 0; j < input_size; j++) {
            hidden[i] += inputs[j] * weights_hidden[i * input_size + j];
        }
        hidden[i] = MathTanh(hidden[i]); // Tanh activation
    }

    // Output layer calculation
    double output = 0;
    for (int i = 0; i < hidden_size; i++) {
        output += hidden[i] * weights_output[i];
    }
    return MathTanh(output); // Tanh activation
}

//+------------------------------------------------------------------+
//| Predict with Python (example integration)                        |
//+------------------------------------------------------------------+
#import "shell32.dll"
int ShellExecuteW(int hwnd, string lpOperation, string lpFile, string lpParameters, string lpDirectory, int nShowCmd);
#import

double PredictWithPython(double &inputs[]) {
    // Save inputs to a file
    int handle = FileOpen("input_data.csv", FILE_WRITE | FILE_CSV);
    if (handle != INVALID_HANDLE) {
        for (int i = 0; i < ArraySize(inputs); i++) {
            FileWrite(handle, inputs[i]);
        }
        FileClose(handle);
    }

    // Execute Python script
    int result = ShellExecuteW(0, "open", "python", "predict.py", "", 0);
    if (result <= 32) {
        Print("Error executing Python script");
        return 0;
    }

    // Read prediction from output file
    handle = FileOpen("output_data.csv", FILE_READ | FILE_CSV);
    if (handle != INVALID_HANDLE) {
        double prediction = FileReadNumber(handle);
        FileClose(handle);
        return prediction;
    }
    return 0;
}
//+------------------------------------------------------------------+
//| Basic trading system using MA                                    |
//+------------------------------------------------------------------+
double basicTradingSystem()
{
    return iMAGet(bar);
}

//+------------------------------------------------------------------+
//| Get MA value                                                    |
//+------------------------------------------------------------------+
double iMAGet(const int index)
{
    double MA[];
    ArraySetAsSeries(MA, true);
    if (CopyBuffer(handle_iMA, 0, 0, index + 2, MA) < 0)
    {
        Print("Failed to copy MA buffer");
        return 0.0;
    }
    double right = MA[index];
    double left = MA[index + 1];
    return right - left;
}

//+------------------------------------------------------------------+
//| Open a trade                                                    |
//+------------------------------------------------------------------+
void OpenTrade(int type)
{
    double price = (type == ORDER_TYPE_BUY) ? SymbolInfoDouble(_Symbol, SYMBOL_ASK) : SymbolInfoDouble(_Symbol, SYMBOL_BID);
    double sl = (type == ORDER_TYPE_BUY) ? price - (m_sl * _Point) : price + (m_sl * _Point);
    double tp = (type == ORDER_TYPE_BUY) ? price + (m_tp * _Point) : price - (m_tp * _Point);

    if (type == ORDER_TYPE_BUY)
        trade.Buy(LotSize, _Symbol, price, sl, tp, "Buy Trade");
    else
        trade.Sell(LotSize, _Symbol, price, sl, tp, "Sell Trade");

    AddTradeLog("Trade opened: " + string(type) + " at " + DoubleToString(price, 5));
}

//+------------------------------------------------------------------+
//| Add trade log entry                                              |
//+------------------------------------------------------------------+
void AddTradeLog(string entry)
{
    TradeLog[TradeLogIndex] = entry;
    TradeLogIndex = (TradeLogIndex + 1) % 100;
}

//+------------------------------------------------------------------+
//| Check if volatility is acceptable                                |
//+------------------------------------------------------------------+
bool IsVolatilityAcceptable()
{
    int atrHandle = iATR(_Symbol, SelectedTimeFrame, ATRPeriod);
    if (atrHandle == INVALID_HANDLE)
    {
        Print("Failed to create ATR handle");
        return false;
    }

    double atrValue[1];
    if (CopyBuffer(atrHandle, 0, 0, 1, atrValue) <= 0)
    {
        Print("Failed to copy ATR buffer");
        return false;
    }

    return atrValue[0] <= 10; // Example threshold
}


//+------------------------------------------------------------------+
//| Check if trend is strong                                         |
//+------------------------------------------------------------------+
bool IsTrendStrong() {
    int adxHandle = iADX(_Symbol, SelectedTimeFrame, ADXThreshold);
    if (adxHandle == INVALID_HANDLE) {
        Print("Failed to create ADX handle");
        return false;
    }

    double adxValue[1];
    if (CopyBuffer(adxHandle, 0, 0, 1, adxValue) <= 0) {
        Print("Failed to copy ADX buffer");
        return false;
    }

    Print("ADX Value: ", adxValue[0], " | Threshold: ", ADXThreshold);
    return adxValue[0] >= ADXThreshold;
}
//+------------------------------------------------------------------+
//| Check if news event is active                                    |
//+------------------------------------------------------------------+
bool IsNewsEventActive()
{
    // Placeholder for news event check
    return false;
}

//+------------------------------------------------------------------+
//| Check if trading time is valid                                   |
//+------------------------------------------------------------------+
bool IsTradingTime()
{
    datetime currentTime = TimeCurrent();
    return (currentTime >= TradingStartTime && currentTime <= TradingEndTime);
}

//+------------------------------------------------------------------+
//| Draw support and resistance levels                               |
//+------------------------------------------------------------------+
void DrawSupportResistance() {
    ObjectsDeleteAll(0, "Support_");
    ObjectsDeleteAll(0, "Resistance_");

if (SwingLows[0] > 0) {
    bool supportCreated = ObjectCreate(0, "Support_Level", OBJ_HLINE, 0, 0, SwingLows[0]);
    if (!supportCreated) {
        Print("Errore nella creazione del livello di supporto: ", GetLastError());
    } else {
        ObjectSetInteger(0, "Support_Level", OBJPROP_COLOR, clrGreen);
        ObjectSetInteger(0, "Support_Level", OBJPROP_WIDTH, 2);
        ObjectSetString(0, "Support_Level", OBJPROP_TEXT, "Support");
    }
} else {
    Print("Errore: Swing Low non valido ", SwingLows[0]);
}

if (SwingHighs[0] > 0) {
    bool resistanceCreated = ObjectCreate(0, "Resistance_Level", OBJ_HLINE, 0, 0, SwingHighs[0]);
    if (!resistanceCreated) {
        Print("Errore nella creazione del livello di resistenza: ", GetLastError());
    } else {
        ObjectSetInteger(0, "Resistance_Level", OBJPROP_COLOR, clrRed);
        ObjectSetInteger(0, "Resistance_Level", OBJPROP_WIDTH, 2);
        ObjectSetString(0, "Resistance_Level", OBJPROP_TEXT, "Resistance");
    }
} else {
    Print("Errore: Swing High non valido ", SwingHighs[0]);
} 
    }



//+------------------------------------------------------------------+
//| Conservative Strategy: Double Bottom and Double Top              |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Check for Double Bottom Pattern                                  |
//+------------------------------------------------------------------+
bool CheckDoubleBottom() {
    // Check if we have two swing lows
    if (SwingLows[0] > 0 && SwingLows[1] > 0) {
        // Calculate termination zone
        terminationZoneHigh = MathMax(iClose(_Symbol, PERIOD_CURRENT, 1), iClose(_Symbol, PERIOD_CURRENT, 2));
        terminationZoneLow = MathMin(iLow(_Symbol, PERIOD_CURRENT, 1), iLow(_Symbol, PERIOD_CURRENT, 2));

        // Check if price touches the termination zone
        double currentLow = iLow(_Symbol, PERIOD_CURRENT, 0);
        if (currentLow >= terminationZoneLow && currentLow <= terminationZoneHigh) {
            // Check for buying pressure (hammer or bullish candle)
            if (IsHammer() || (iClose(_Symbol, PERIOD_CURRENT, 0) > iOpen(_Symbol, PERIOD_CURRENT, 0))) {
                // Add volume filter
                long volume = iVolume(_Symbol, PERIOD_CURRENT, 0); // Usa long invece di double
                if (volume > iVolume(_Symbol, PERIOD_CURRENT, 1)) {
                    // Add RSI filter (oversold condition)
                    double rsiBuffer[];
                    ArraySetAsSeries(rsiBuffer, true);
                    int rsiHandle = iRSI(_Symbol, PERIOD_CURRENT, RSI_Period, PRICE_CLOSE);
                    if (CopyBuffer(rsiHandle, 0, 0, 1, rsiBuffer) <= 0)
                    {
                        Print("Failed to copy RSI buffer");
                        return false;
                    }
                    double rsi = rsiBuffer[0];
                    if (rsi < 30) {
                        isDoubleBottom = true;
                        return true;
                    }
                }
            }
        }
    }
    return false;
}

//+------------------------------------------------------------------+
//| Check for Double Top Pattern                                     |
//+------------------------------------------------------------------+
bool CheckDoubleTop() {
    // Check if we have two swing highs
    if (SwingHighs[0] > 0 && SwingHighs[1] > 0) {
        // Calculate termination zone
        terminationZoneHigh = MathMax(iHigh(_Symbol, PERIOD_CURRENT, 1), iHigh(_Symbol, PERIOD_CURRENT, 2));
        terminationZoneLow = MathMin(iClose(_Symbol, PERIOD_CURRENT, 1), iClose(_Symbol, PERIOD_CURRENT, 2));

        // Check if price touches the termination zone
        double currentHigh = iHigh(_Symbol, PERIOD_CURRENT, 0);
        if (currentHigh >= terminationZoneLow && currentHigh <= terminationZoneHigh) {
            // Check for selling pressure (shooting star or bearish candle)
            if (IsShootingStar() || (iClose(_Symbol, PERIOD_CURRENT, 0) < iOpen(_Symbol, PERIOD_CURRENT, 0))) {
                // Add volume filter
                long volume = iVolume(_Symbol, PERIOD_CURRENT, 0); // Usa long invece di double
                if (volume > iVolume(_Symbol, PERIOD_CURRENT, 1)) {
                    // Add RSI filter (overbought condition)
                    double rsiBuffer[];
                    ArraySetAsSeries(rsiBuffer, true);
                    int rsiHandle = iRSI(_Symbol, PERIOD_CURRENT, RSI_Period, PRICE_CLOSE);
                    if (CopyBuffer(rsiHandle, 0, 0, 1, rsiBuffer) <= 0)
                    {
                        Print("Failed to copy RSI buffer");
                        return false;
                    }
                    double rsi = rsiBuffer[0];
                    if (rsi > 70) {
                        isDoubleTop = true;
                        return true;
                    }
                }
            }
        }
    }
    return false;
}

//+------------------------------------------------------------------+
//| Open Trade for Conservative Strategy                             |
//+------------------------------------------------------------------+
void OpenConservativeTrade() {
    if (isDoubleBottom) {
        double entryPrice = iClose(_Symbol, PERIOD_CURRENT, 0);
        double stopLoss = terminationZoneLow - (10 * _Point); // Stop loss below the lowest low
        double takeProfit = entryPrice + (1.4 * (entryPrice - stopLoss)); // 1:1.4 risk-reward ratio

        trade.Buy(LotSize, _Symbol, entryPrice, stopLoss, takeProfit, "Double Bottom Trade");
        isDoubleBottom = false; // Reset flag
    }

    if (isDoubleTop) {
        double entryPrice = iClose(_Symbol, PERIOD_CURRENT, 0);
        double stopLoss = terminationZoneHigh + (10 * _Point); // Stop loss above the highest high
        double takeProfit = entryPrice - (1.4 * (stopLoss - entryPrice)); // 1:1.4 risk-reward ratio

        trade.Sell(LotSize, _Symbol, entryPrice, stopLoss, takeProfit, "Double Top Trade");
        isDoubleTop = false; // Reset flag
    }
}

//+------------------------------------------------------------------+
//| Check for Hammer Pattern                                         |
//+------------------------------------------------------------------+
bool IsHammer() {
    double open = iOpen(_Symbol, PERIOD_CURRENT, 0);
    double close = iClose(_Symbol, PERIOD_CURRENT, 0);
    double low = iLow(_Symbol, PERIOD_CURRENT, 0);
    double high = iHigh(_Symbol, PERIOD_CURRENT, 0);

    double body = MathAbs(open - close);
    double lowerShadow = low - MathMin(open, close);
    double upperShadow = MathMax(open, close) - high;

    // Fibonacci retracement check (38.2%)
    double fibLevel = low + (high - low) * 0.382;
    bool isBodyAboveFib = (MathMin(open, close) > fibLevel);

    // Add RSI filter (oversold condition)
    double rsiBuffer[];
    ArraySetAsSeries(rsiBuffer, true);
    int rsiHandle = iRSI(_Symbol, PERIOD_CURRENT, RSI_Period, PRICE_CLOSE);
    if (CopyBuffer(rsiHandle, 0, 0, 1, rsiBuffer) <= 0)
    {
        Print("Failed to copy RSI buffer");
        return false;
    }
    double rsi = rsiBuffer[0];
    bool isRSIOversold = (rsi < 30);

    // Add MACD filter (bullish crossover)
    double macdBuffer[];
    double signalBuffer[];
    ArraySetAsSeries(macdBuffer, true);
    ArraySetAsSeries(signalBuffer, true);

    int macdHandle = iMACD(_Symbol, PERIOD_CURRENT, MACD_Fast, MACD_Slow, MACD_Signal, PRICE_CLOSE);
    if (CopyBuffer(macdHandle, 0, 0, 1, macdBuffer) <= 0 || CopyBuffer(macdHandle, 1, 0, 1, signalBuffer) <= 0)
    {
        Print("Failed to copy MACD buffers");
        return false;
    }

    double macd = macdBuffer[0]; // MACD Line
    double signal = signalBuffer[0]; // Signal Line
    bool isMACDBullish = (macd > signal);

    return (lowerShadow > 2 * body) && (upperShadow < 0.3 * lowerShadow) && isBodyAboveFib && isRSIOversold && isMACDBullish;
}

//+------------------------------------------------------------------+
//| Check for Shooting Star Pattern                                  |
//+------------------------------------------------------------------+
bool IsShootingStar() {
    double open = iOpen(_Symbol, PERIOD_CURRENT, 0);
    double close = iClose(_Symbol, PERIOD_CURRENT, 0);
    double low = iLow(_Symbol, PERIOD_CURRENT, 0);
    double high = iHigh(_Symbol, PERIOD_CURRENT, 0);

    double body = MathAbs(open - close);
    double lowerShadow = low - MathMin(open, close);
    double upperShadow = MathMax(open, close) - high;

    // Fibonacci retracement check (38.2%)
    double fibLevel = low + (high - low) * 0.382;
    bool isBodyBelowFib = (MathMax(open, close) < fibLevel);

    // Add RSI filter (overbought condition)
    double rsiBuffer[];
    ArraySetAsSeries(rsiBuffer, true);
    int rsiHandle = iRSI(_Symbol, PERIOD_CURRENT, RSI_Period, PRICE_CLOSE);
    if (CopyBuffer(rsiHandle, 0, 0, 1, rsiBuffer) <= 0)
    {
        Print("Failed to copy RSI buffer");
        return false;
    }
    double rsi = rsiBuffer[0];
    bool isRSIOverbought = (rsi > 70);

    // Add MACD filter (bearish crossover)
    double macdBuffer[];
    double signalBuffer[];
    ArraySetAsSeries(macdBuffer, true);
    ArraySetAsSeries(signalBuffer, true);

    int macdHandle = iMACD(_Symbol, PERIOD_CURRENT, MACD_Fast, MACD_Slow, MACD_Signal, PRICE_CLOSE);
    if (CopyBuffer(macdHandle, 0, 0, 1, macdBuffer) <= 0 || CopyBuffer(macdHandle, 1, 0, 1, signalBuffer) <= 0)
    {
        Print("Failed to copy MACD buffers");
        return false;
    }

    double macd = macdBuffer[0]; // MACD Line
    double signal = signalBuffer[0]; // Signal Line
    bool isMACDBearish = (macd < signal);

    return (upperShadow > 2 * body) && (lowerShadow < 0.3 * upperShadow) && isBodyBelowFib && isRSIOverbought && isMACDBearish;
}
bool IsConfirmationCandle(int type) {
    if (type == ORDER_TYPE_BUY) {
        // Check for green candle after hammer
        return iClose(_Symbol, PERIOD_CURRENT, 0) > iOpen(_Symbol, PERIOD_CURRENT, 0);
    } else if (type == ORDER_TYPE_SELL) {
        // Check for red candle after shooting star
        return iClose(_Symbol, PERIOD_CURRENT, 0) < iOpen(_Symbol, PERIOD_CURRENT, 0);
    }
    return false;
}
